import { FetchOptions } from 'ohmyfetch';

interface ResponseMap {
    blob: Blob;
    text: string;
    arrayBuffer: ArrayBuffer;
}
declare type ResponseType = keyof ResponseMap | "json";
declare type MappedType<R extends ResponseType, JsonType = any> = R extends keyof ResponseMap ? ResponseMap[R] : JsonType;
declare type ClientMethodHandler = <T = any, R extends ResponseType = "json">(data?: RequestInit["body"] | Record<string, any>, opts?: Omit<FetchOptions<R>, "baseURL" | "method">) => Promise<MappedType<R, T>>;
declare type ClientBuilder = {
    [key: string]: ClientBuilder;
    (...segmentsOrIds: (string | number)[]): ClientBuilder;
} & {
    get: ClientMethodHandler;
    post: ClientMethodHandler;
    put: ClientMethodHandler;
    delete: ClientMethodHandler;
    patch: ClientMethodHandler;
};

/**
 * Minimal, type-safe REST client using JS proxies
 */
declare function createClient<R extends ResponseType = "json">(url: string, defaults?: Omit<FetchOptions<R>, "method">): ClientBuilder;

export { ClientBuilder, createClient };
